/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WARDeploymentDescLoader.java 6176 2012-02-21 13:25:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class analyzes the WAR deployment descriptor and fill a struct with
 * getters.
 *
 * @author Florent Benoit
 */
public class WARDeploymentDescLoader extends AbsWebLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARDeploymentDescLoader.class);


    /**
     * Constructor.
     *
     * @param url the url that is pointing to the web xml file.
     */
    protected WARDeploymentDescLoader(final URL url) {
        super(url);
    }

    /**
     * Load the web.xml file.
     *
     * @param url the URL of the XML file.
     * @param validation true for forcong validation
     * @return an application object.
     * @throws ParsingException if parsing of XML file fails.
     */
    public static WAR loadDeploymentDescriptor(final URL url, boolean validation) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        WARDeploymentDescLoader warDeploymentDescLoader = new WARDeploymentDescLoader(url);
        warDeploymentDescLoader.setValidating(validation);
        warDeploymentDescLoader.parse();
        return warDeploymentDescLoader.getWAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     *
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {

        // No parsing without WEB-INF/web.xml file
        if (getURL() == null) {
            return;
        }
        
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new WAREntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <web-app>
        Element webAppRootElement = document.getDocumentElement();

        if (webAppRootElement.getNamespaceURI() != null) {
            // Define NS
            defineXMLNS(webAppRootElement);
        }

        analyzeEnvironment(webAppRootElement, getWAR());

        // Find and analyze the servlets
        analyzeServlets(webAppRootElement);

        // and listeners
        analyzeListeners(webAppRootElement);

        // and filters
        analyzeFilters(webAppRootElement);


        String metadataComplete = webAppRootElement.getAttribute("metadata-complete");
        if (metadataComplete.equals("true")) {
            getWAR().setMetadataComplete(true);
        }
        if (isValidating()) {
            // vericonformity of some elements
            if (!analyzeJspConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of jsp-config");
            }
            if (!analyzeSessionConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of session-config");
            }
            if (!analyzeLoginConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of login-config");
            }
            if (!analyzeUrlPattern(webAppRootElement)) {
                throw new ParsingException(" a '\\n' character inside the url pattern is forbidden");
            }
        }
    }


}
